@@ -234,7 +234,9 @@ def read_training_validation_from_COS(model_id,icos_client,bucketName,bucketName
     print ('test Data=',testData.shape)
     return feature_data_original,feature_data_normal,validate_df,validation_wo_normal_df,testData
 #----------------------------------------------------------------
-
+#
+# add anomaly score as one column of feature csv file, and save it to COS
+#----------------------------------------------------------------
 def print_and_plot_anomaly_score(model_name,df_join_TF,Problem_object,anomaly_threshold,anomaly_score,percentile_point_95,plot_anomaly_score='YES'):
     # get from problem report:
     DR_file_df = Problem_object['DR_file_df']
@@ -290,4 +292,36 @@ def print_and_plot_anomaly_score(model_name,df_join_TF,Problem_object,anomaly_th
     print('=========number of anomaly =============== \n percentile_95:',num_anomaly_95,', bad validation num above threshold:',num_anomaly_threshold)
     print('---------number of normal ---------------- \n num_normal_green_list:',num_normal_green_list,', total num_normal_below_threshold:',num_normal_below_threshold)
     print(num_anomaly_df)
-    return num_anomaly_95,num_anomaly_threshold
\ No newline at end of file
+    return num_anomaly_95,num_anomaly_threshold
+
+#----------- anomaly score -----------------------
+def get_anomaly_score(model_id,icos_client,bucketName,pipeline,selected_features,testData_date,localPath='./Production_Anomaly_data/'):
+    deviceId = model_id.split('_weld')[0] # Formet_FR4_STA60_LH_R1_weld32_toolB
+    weldId_with_toolID =  model_id.split('_weld')[1]
+    feature_file_name = 'Production_Lincoln_features_data/'+deviceId+'_LincolnFANUC_'+testData_date+'_welding_stable_data_weldid_'+weldId_with_toolID+'_feature.csv'
+    body = icos_client.get_object(Bucket=bucketName,Key=feature_file_name)['Body']
+    # add missing __iter__ method, so pandas accepts body as file-like object
+    if not hasattr(body, "__iter__"): body.__iter__ = types.MethodType( __iter__, body )
+    feature_data = pd.read_csv(body)
+    
+    scoring_data = feature_data[selected_features]
+    
+    ###########################################################################  
+    anomaly_score_initial = pipeline.predict_proba(scoring_data) 
+    ###########################################################################      
+    #anomaly_threshold_initial = pipeline.get_best_thresholds()
+
+    anomaly_score = pd.DataFrame(anomaly_score_initial)
+    df_anomaly_score = pd.DataFrame(anomaly_score)            
+    feature_data['anomaly_score'] = anomaly_score
+    #
+    # --- write the scoring result to csv file
+    #
+    fileName = model_id+'_anomaly_score_'+testData_date+'.csv'
+    localfileName=localPath+fileName
+    feature_data.to_csv(localfileName,index=False)
+    # write to COS 
+    csv_fileName='Production_Features_and_Anomaly_Score_data/'+ fileName
+    icos_client.upload_file(Filename=localfileName,Bucket=bucketName,Key=csv_fileName)
+    print('write to COS:'+csv_fileName)
+    return feature_data
